﻿using System;
using System.Collections.Generic;
using System.Linq;
using Foundation;
using UIKit;
using Xamarin.Forms;
using LoadFromGallerySample.iOS;
using ZXing;
using ZXing.Common;
using System.IO;
using CoreGraphics;

[assembly: Dependency(typeof(LoadImageiOS))]
namespace LoadFromGallerySample.iOS
{
    // The UIApplicationDelegate for the application. This class is responsible for launching the 
    // User Interface of the application, as well as listening (and optionally responding) to 
    // application events from iOS.
    [Register("AppDelegate")]
    public partial class AppDelegate : global::Xamarin.Forms.Platform.iOS.FormsApplicationDelegate
    {
        //
        // This method is invoked when the application has loaded and is ready to run. In this 
        // method you should instantiate the window, load the UI into it and then make the window
        // visible.
        //
        // You have 17 seconds to return from this method, or iOS will terminate your application.
        //
        public override bool FinishedLaunching(UIApplication app, NSDictionary options)
        {
            global::Xamarin.Forms.Forms.Init();
            LoadApplication(new App());
          
            return base.FinishedLaunching(app, options);
        }
    }

    public class LoadImageiOS : ILoad
    {
        UIImagePickerController imagePicker;

        void ILoad.UploadFromGallery(MainPage editor)
        {
            imagePicker = new UIImagePickerController();
            imagePicker.AllowsEditing = true;
            imagePicker.SourceType = UIImagePickerControllerSourceType.PhotoLibrary;
            imagePicker.MediaTypes = UIImagePickerController.AvailableMediaTypes(UIImagePickerControllerSourceType.PhotoLibrary);
            imagePicker.FinishedPickingMedia += (object sender, UIImagePickerMediaPickedEventArgs e) =>
            {
                imagePicker.DismissModalViewController(true);
                var image = e.OriginalImage ?? e.EditedImage;
               
                editor.SwitchView(image.AsPNG().AsStream());
            };
            imagePicker.Canceled += (sender, evt) =>
            {
                imagePicker.DismissModalViewController(true);
            };
            var topController = UIApplication.SharedApplication.KeyWindow.RootViewController;
            while (topController.PresentedViewController != null)
            {
                topController = topController.PresentedViewController;
            }
            topController.PresentViewController(imagePicker, true, null);
        }

        /// <summary>
        /// Get binary bitmap image from image stream to read barcode
        /// </summary>
        /// <param name="imageBytes"></param>
        /// <returns></returns>
        public BinaryBitmap GetBinaryBitmap(byte[] imageBytes)
        {
            Stream imageStream = new MemoryStream(imageBytes);
            var imageData = NSData.FromStream(imageStream);

            UIImageView imgView = new UIImageView();
            UIImage image = new UIImage(imageData);
           
            //First get the image into your data buffer
            CGImage imageRef = image.CGImage;
            string width = imageRef.Width.ToString();
            var w = Convert.ToInt32(width);
            string height = imageRef.Height.ToString();
            var h = Convert.ToInt32(height);

            byte[] rgbBytes = GetRgbBytes(image);

            var bin = new HybridBinarizer(new RGBLuminanceSource(rgbBytes, w, h));
            var i = new BinaryBitmap(bin);

            return i;
        }

        /// <summary>
        /// Get RGB value from image 
        /// </summary>
        /// <param name="image"></param>
        /// <returns></returns>
        private byte[] GetRgbBytes(UIImage image)
        {
            var rgbBytes = new List<byte>();

            NSMutableArray<UIColor> result = new NSMutableArray<UIColor>();

            // First get the image into your data buffer
            CGImage imageRef = image.CGImage;
            nint width = imageRef.Width;
            nint height = imageRef.Height;
            CGColorSpace colorSpace = CGColorSpace.CreateDeviceRGB();
            byte[] rawData = new byte[height * width * 4];
            nint bytesPerPixel = 4;
            nint bytesPerRow = bytesPerPixel * width;
            nint bitsPerComponent = 8;
            CGContext context = new CGBitmapContext(rawData, width, height, bitsPerComponent, bytesPerRow, colorSpace, CGBitmapFlags.ByteOrder32Big | CGBitmapFlags.PremultipliedLast);
            colorSpace.Dispose();

            context.DrawImage(new CGRect(0, 0, width, height), imageRef);
            context.Dispose();

            // Now your rawData contains the image data in the RGBA8888 pixel format.
            nint byteIndex = 0;

            string count = (image.Size.Width * image.Size.Height).ToString();
            int c = Convert.ToInt32(count);

            for (int i = 0; i < c; ++i)
            {
                byte alpha = rawData[byteIndex + 3];
                byte red = rawData[byteIndex];
                byte green = rawData[byteIndex + 1];
                byte blue = rawData[byteIndex + 2];
                byteIndex += bytesPerPixel;

                UIColor acolor = new UIColor(red, green, blue, alpha);
                result.Add(acolor);

                rgbBytes.AddRange(new[] { red, green, blue });
            }

            return rgbBytes.ToArray();
        }
    }
}
